# This file is part of the digitalSTROM dsm-api library.
#
# Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland
#
# digitalSTROM dsm-api library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# digitalSTROM dsm-api library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with digitalSTROM dsm-api library. If not, see <http://www.gnu.org/licenses/>.

from SwigGenerator import *
from string import Template

def getPyDoc(self):
    return self.name + " -- " + self.description
Parameter.getPyDoc = getPyDoc

def getPyTypeString(self):
    if self.type_size == 8:
        if (self.type_count == 1):
            return "B"
        elif (self.name == "Name" and self.type_count == 21):
            return "s"
        else:
            return "[" + "B" * self.type_count + "]"
    elif self.type_size == 16:
        return "H"
    elif self.type_size == 32:
        return "I"
Parameter.getPyTypeString = getPyTypeString

def getPyCall(self):
    if (self.type_count == 1):
        return self.name
    else:
        if self.name == "DSID" and self.type_size == 8 and self.type_count == 12:
            return ", ".join(map(lambda x: "DSID.id[" + str(x) + "]", range(self.type_count)))
        elif self.name == "dSUID" and self.type_size == 8 and self.type_count == 17:
            return ", ".join(map(lambda x: "dSUID.id[" + str(x) + "]", range(self.type_count)))
        elif (self.name == "Name" and self.type_count == 21):
            return self.name
        else:
            return ", ".join(map(lambda x: self.name + "[" + str(x) + "]", range(self.type_count)))
Parameter.getPyCall = getPyCall


def py_callback_implementation(self, type):
    s = Template("""void ${name}_${type}_py_callback(uint8_t error_code, void *arg, dsuid_t sourceId, dsuid_t destinationId${parameter_decl})
{
    PyObject *arglist;
    PyObject *result;
    PyObject* python_callback = (PyObject*)arg;
    
    SWIG_PYTHON_THREAD_BEGIN_BLOCK;
    
    arglist = Py_BuildValue("(B${type_string})", error_code, sourceId.id[0], sourceId.id[1], sourceId.id[2], sourceId.id[3],
                                                sourceId.id[4], sourceId.id[5], sourceId.id[6], sourceId.id[7],
                                                sourceId.id[8], sourceId.id[9], sourceId.id[10], sourceId.id[11],
                                                sourceId.id[12], sourceId.id[13], sourceId.id[14], sourceId.id[15],
                                                sourceId.id[16],
                                                destinationId.id[0], destinationId.id[1], destinationId.id[2], destinationId.id[3],
                                                destinationId.id[4], destinationId.id[5], destinationId.id[6], destinationId.id[7],
                                                destinationId.id[8], destinationId.id[9], destinationId.id[10], destinationId.id[11],
                                                destinationId.id[12], destinationId.id[13], destinationId.id[14], destinationId.id[15],
                                                destinationId.id[16]
                                                ${param_call});
    
    result = PyEval_CallObject(python_callback, arglist);
    
    Py_XDECREF(arglist);
    Py_XDECREF(result);
    
    SWIG_PYTHON_THREAD_END_BLOCK;
}
""")
    if (type == "request"):
        parameters = self.request_params
    else:
        parameters = self.response_params
    
    parameter_decl = ""
    param_call = ""
    type_string = ("[" + "B" * 17 + "]") * 2
    if parameters:
        parameter_decl = ", " + ", ".join(map(lambda x: x.signature(), parameters))
        param_call = ", " + ", ".join(map(lambda x: x.getPyCall(), parameters))
        type_string += "".join(map(lambda x: x.getPyTypeString(), parameters))
        
    return s.substitute(name = self.name,
                        type = type,
                        parameter_decl = parameter_decl,
                        type_string = type_string,
                        param_call = param_call)
ApiCall.py_callback_implementation = py_callback_implementation

def pyCallbackRegisterSelector(self, call_type):
    s = Template("""    if (MessageType == ${type_const} && MessageId == ${message_id} && ModifierId == ${modifier_id})
    {
        DsmApiCallback_t callback, oldcallback;
        callback.function = ${name}_${type}_py_callback;
        callback.arg = pyfunc;
        if (DsmApiSetCallback(handle, MessageType, MessageId, ModifierId, &callback, &oldcallback) == ERROR_OK)
        {
            Py_XINCREF(pyfunc);
            Py_XDECREF(oldcallback.arg);
        }
        return;
    }""")
    
    if call_type == "response":
        type_const = "DS485_CONTAINER_RESPONSE"
    elif call_type == "request":
        type_const = "DS485_CONTAINER_REQUEST"
    else:
        type_const = "DS485_CONTAINER_EVENT"
        
    return s.substitute(message_id = str(self.message_id),
                        modifier_id = str(self.modifier_id),
                        name = self.name,
                        type = call_type,
                        type_const = type_const)
ApiCall.pyCallbackRegisterSelector = pyCallbackRegisterSelector

def py_docstring(self):
    s = Template("""%feature("autodoc", "${name}(${inParameters}) -> (${outParameters})") ${name};
%feature("docstring") "$description
$vdsm_only_warning

This call has a timeout of $timeout seconds.

Arguments:
$request

Return values:
$response" """)
    return s.substitute(name = self.name,
                        inParameters = ", ".join(["handle", "dSMdSUID"] + map(lambda x: x.name, self.request_params)),
                        outParameters = ", ".join(["error"] + map(lambda x: x.name, self.response_params)),
                        description = self.description,
                        vdsm_only_warning = "This function can be used only within a vDsm!" if self.long_message else "",
                        timeout = self.timeout,
                        request = "\n".join(["handle -- Handle to the library instance", "dSMdSUID -- Destination of the call"] + map(lambda x: x.getPyDoc(), self.request_params)),
                        response = "\n".join(["error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code"] + map(lambda x: x.getPyDoc(), self.response_params)))
ApiCall.py_docstring = py_docstring

def swig_implementation(self):
    s = ""
    for call in [x for x in self.calls if not x.deprecated]:
        s += call.py_callback_implementation("request") + "\n"
        s += call.py_callback_implementation("response") + "\n"
    for event in [x for x in self.events if not x.deprecated]:
        s += event.py_callback_implementation("event") + "\n"
    return s
CallGroup.swig_implementation = swig_implementation

def swig_callback_selector(self):
    s = ""
    for call in [x for x in self.calls if not x.deprecated]:
        s += call.pyCallbackRegisterSelector("request") + "\n\n"
        s += call.pyCallbackRegisterSelector("response") + "\n\n"
    for event in [x for x in self.events if not x.deprecated]:
        s += event.pyCallbackRegisterSelector("event") + "\n\n"
    return s
CallGroup.swig_callback_selector = swig_callback_selector

def swig_declaration(self):
    s = ""
    for call in [x for x in self.calls if not x.deprecated]:
        s += call.py_docstring() + "\n"
        s += call.call_signature_swig() + ";\n"
    return s
CallGroup.swig_declaration = swig_declaration

class PyGenerator:
    def __init__(self, license, groups):
        self.groups = groups
        self.license = license
        
    def swig(self, swig_decl_file, length_enum):
        swig = open(swig_decl_file, 'w')
        swig_str = self.license
        swig_str += """%module dsmapi2
%{
#include "dsm-api-const.h"
static int convert_uint8_t(PyObject *input, uint8_t *ptr, int size) {
  int i;
  if (!PySequence_Check(input)) {
      PyErr_SetString(PyExc_TypeError,"Expecting a sequence");
      return 0;
  }
  if (PyObject_Length(input) != size) {
      PyErr_SetString(PyExc_ValueError,"Sequence size mismatch");
      return 0;
  }
  for (i =0; i < size; i++) {
      PyObject *o = PySequence_GetItem(input,i);
      if (!PyInt_Check(o)) {
         Py_XDECREF(o);
         PyErr_SetString(PyExc_ValueError,"Expecting a sequence of integers");
         return 0;
      }
      ptr[i] = PyInt_AsLong(o);
      Py_DECREF(o);
  }
  return 1;
}
static int convert_uint8_t_max_size(PyObject *input, uint8_t *ptr, int size) {
  int i;
  if (!PySequence_Check(input)) {
      PyErr_SetString(PyExc_TypeError,"Expecting a sequence");
      return 0;
  }
  if (PyObject_Length(input) > size) {
      PyErr_SetString(PyExc_ValueError,"Sequence size mismatch");
      return 0;
  }
  for (i =0; i < PyObject_Length(input); i++) {
      PyObject *o = PySequence_GetItem(input,i);
      if (!PyInt_Check(o)) {
         Py_XDECREF(o);
         PyErr_SetString(PyExc_ValueError,"Expecting a sequence of integers");
         return 0;
      }
      ptr[i] = PyInt_AsLong(o);
      Py_DECREF(o);
  }
  return 1;
}
static int convert_string(PyObject *input, uint8_t *ptr, int size) {
  int i;
  if (!PySequence_Check(input)) {
      PyErr_SetString(PyExc_TypeError,"Expecting a sequence");
      return 0;
  }
  if (PyObject_Length(input) > (size - 1)) {
      PyErr_SetString(PyExc_ValueError,"Sequence size mismatch");
      return 0;
  }
  if (!PyString_Check(input))
  {
      PyErr_SetString(PyExc_ValueError,"Expecting a string");
      return 0;
  }
  memcpy(ptr, PyString_AsString(input), PyString_Size(input));
  memset(ptr + PyString_Size(input), 0, size-PyString_Size(input));
  return 1;
}

static void convert_uint8_array_to_pylist(PyObject **result, uint8_t *data, uint16_t length)
{
    int i;
    PyObject *o, *o2, *o3;
    o = PyList_New(length);
    for (i = 0; i < length; ++i) {
        PyObject *item = PyInt_FromLong(data[i]);
        PyList_SetItem(o, i, item);
    }
    if ((!*result) || (*result == Py_None)) {
        *result = o;
    } else {
        if (!PyTuple_Check(*result)) {
            PyObject *o2 = *result;
            *result = PyTuple_New(1);
            PyTuple_SetItem(*result, 0, o2);
        }
        o3 = PyTuple_New(1);
        PyTuple_SetItem(o3, 0, o);
        o2 = *result;
        *result = PySequence_Concat(o2, o3);
        Py_DECREF(o2);
        Py_DECREF(o3);
    }
}

static void convert_uint8_array_to_pylist_max_size(PyObject **result, uint8_t *data, uint16_t *size, uint16_t max_size)
{
    int i;
    PyObject *o, *o2, *o3;
    uint16_t length;
    if (*size > max_size) {
        length = max_size;
    } else {
        length = *size;
    }
    o = PyList_New(length);
    for (i = 0; i < length; ++i) {
        PyObject *item = PyInt_FromLong(data[i]);
        PyList_SetItem(o, i, item);
    }
    if ((!*result) || (*result == Py_None)) {
        *result = o;
    } else {
        if (!PyTuple_Check(*result)) {
            PyObject *o2 = *result;
            *result = PyTuple_New(1);
            PyTuple_SetItem(*result, 0, o2);
        }
        o3 = PyTuple_New(1);
        PyTuple_SetItem(o3, 0, o);
        o2 = *result;
        *result = PySequence_Concat(o2, o3);
        Py_DECREF(o2);
        Py_DECREF(o3);
    }
}

static void convert_uint8_to_pylong(PyObject **result, uint8_t *data)
{
    PyObject *o, *o2, *o3;
    o = PyInt_FromLong(*data);
    if ((!*result) || (*result == Py_None)) {
        *result = o;
    } else {
        if (!PyTuple_Check(*result)) {
            PyObject *o2 = *result;
            *result = PyTuple_New(1);
            PyTuple_SetItem(*result, 0, o2);
        }
        o3 = PyTuple_New(1);
        PyTuple_SetItem(o3, 0, o);
        o2 = *result;
        *result = PySequence_Concat(o2, o3);
        Py_DECREF(o2);
        Py_DECREF(o3);
    }
}

static void convert_uint16_to_pylong(PyObject **result, uint16_t *data)
{
    PyObject *o, *o2, *o3;
    o = PyInt_FromLong(*data);
    if ((!*result) || (*result == Py_None)) {
        *result = o;
    } else {
        if (!PyTuple_Check(*result)) {
            PyObject *o2 = *result;
            *result = PyTuple_New(1);
            PyTuple_SetItem(*result, 0, o2);
        }
        o3 = PyTuple_New(1);
        PyTuple_SetItem(o3, 0, o);
        o2 = *result;
        *result = PySequence_Concat(o2, o3);
        Py_DECREF(o2);
        Py_DECREF(o3);
    }
}

static void convert_uint32_to_pylong(PyObject **result, uint32_t *data)
{
    PyObject *o, *o2, *o3;
    o = PyLong_FromUnsignedLong(*data);
    if ((!*result) || (*result == Py_None)) {
        *result = o;
    } else {
        if (!PyTuple_Check(*result)) {
            PyObject *o2 = *result;
            *result = PyTuple_New(1);
            PyTuple_SetItem(*result, 0, o2);
        }
        o3 = PyTuple_New(1);
        PyTuple_SetItem(o3, 0, o);
        o2 = *result;
        *result = PySequence_Concat(o2, o3);
        Py_DECREF(o2);
        Py_DECREF(o3);
    }
}

static void convert_uint8_attay_to_pystring(PyObject **result, uint8_t *data)
{
    PyObject *o, *o2, *o3;
    if (data != NULL) {
        o = PyString_FromString(data);
    } else {
        o = PyString_FromString("");
    }
    if ((!*result) || (*result == Py_None)) {
        *result = o;
    } else {
        if (!PyTuple_Check(*result)) {
            PyObject *o2 = *result;
            *result = PyTuple_New(1);
            PyTuple_SetItem(*result, 0, o2);
        }
        o3 = PyTuple_New(1);
        PyTuple_SetItem(o3, 0, o);
        o2 = *result;
        *result = PySequence_Concat(o2, o3);
        Py_DECREF(o2);
        Py_DECREF(o3);
    }
}

/* Includes the header in the wrapper code */
#include "dsm-api.h"
"""
        for group in self.groups:
            swig_str += group.swig_implementation()
            
        swig_str += """
static void BusChange_callback(void *arg, dsuid_t *id, int flag)
{
    PyObject *arglist;
    PyObject *result;
    PyObject* python_callback = (PyObject*)arg;
    
    SWIG_PYTHON_THREAD_BEGIN_BLOCK;
    
    arglist = Py_BuildValue("([BBBBBBBBBBBBBBBBB]B)", id->id[0], id->id[1], id->id[2], id->id[3],
                                                      id->id[4], id->id[5], id->id[6], id->id[7],
                                                      id->id[8], id->id[9], id->id[10], id->id[11],
                                                      id->id[12], id->id[13], id->id[14], id->id[15],
                                                      id->id[16],
                                                      flag);
    
    result = PyEval_CallObject(python_callback, arglist);
    
    Py_XDECREF(arglist);
    Py_XDECREF(result);
    
    SWIG_PYTHON_THREAD_END_BLOCK;
}

void DsmApiRegisterPyBusChangeCallback(DsmApiHandle_t handle, PyObject *pyfunc)
{
    DsmApiCallback_t callback, oldcallback;

    if (!PyCallable_Check(pyfunc))
    {
        callback.function = 0;
        callback.arg = 0;
        if (DsmApiSetBusChangeCallback(handle, &callback, &oldcallback) == ERROR_OK)
        {
            Py_XDECREF(oldcallback.arg);
        }
        return;
    }

    callback.function = &BusChange_callback;
    callback.arg = pyfunc;
    
    if (DsmApiSetBusChangeCallback(handle, &callback, &oldcallback) == ERROR_OK)
    {
        Py_INCREF(pyfunc);
        Py_XDECREF(oldcallback.arg);
    }
}

static void LinkState_callback(void *arg, bool _state)
{
    PyObject *arglist;
    PyObject *result;
    PyObject* python_callback = (PyObject*)arg;
    
    SWIG_PYTHON_THREAD_BEGIN_BLOCK;
    
    arglist = Py_BuildValue("(B)", _state);
    result = PyEval_CallObject(python_callback, arglist);
    
    Py_XDECREF(arglist);
    Py_XDECREF(result);
    
    SWIG_PYTHON_THREAD_END_BLOCK;
}

void DsmApiRegisterPyLinkStateCallback(DsmApiHandle_t handle, PyObject *pyfunc)
{
    DsmApiCallback_t callback, oldcallback;

    if (!PyCallable_Check(pyfunc))
    {
        callback.function = 0;
        callback.arg = 0;
        if (DsmApiSetLinkStateCallback(handle, &callback, &oldcallback) == ERROR_OK)
        {
            Py_XDECREF(oldcallback.arg);
        }
        return;
    }

    callback.function = &LinkState_callback;
    callback.arg = pyfunc;
    
    if (DsmApiSetLinkStateCallback(handle, &callback, &oldcallback) == ERROR_OK)
    {
        Py_INCREF(pyfunc);
        Py_XDECREF(oldcallback.arg);
    }
}
void DsmApiRegisterPyCallback(DsmApiHandle_t handle, container_type_t MessageType, int MessageId, int ModifierId, PyObject *pyfunc)
{
    if (!PyCallable_Check(pyfunc))
    {
        DsmApiCallback_t callback, oldcallback;
        callback.function = 0;
        callback.arg = 0;
        if (DsmApiSetCallback(handle, MessageType, MessageId, ModifierId, &callback, &oldcallback) == ERROR_OK)
        {
            Py_XDECREF(oldcallback.arg);
        }
        return;
    }
"""
    
        for group in self.groups:
            swig_str += group.swig_callback_selector()
        
        swig_str += """
}
%}

%typemap(in,numinputs=0) uint8_t *Out(uint8_t temp) {
    $1 = &temp;
}

%typemap(in) uint8_t {
    $1 = PyInt_AsLong($input);
}

%typemap(argout) uint8_t *Out {
    convert_uint8_to_pylong(&$result, $1);
}

%typemap(in) uint16_t {
    $1 = PyInt_AsLong($input);
}

%typemap(in,numinputs=0) uint16_t *Out(uint16_t temp) {
    $1 = &temp;
}

%typemap(argout) uint16_t *Out {
    convert_uint16_to_pylong(&$result, $1);
}

%typemap(in) uint32_t {
    $1 = PyLong_AsUnsignedLong($input);
}

%typemap(in,numinputs=0) uint32_t *Out(uint32_t temp) {
    $1 = &temp;
}

%typemap(argout) uint32_t *Out {
    convert_uint32_to_pylong(&$result, $1);
}

%typemap(in,numinputs=0) (dsid_t *dsid)(uint8_t temp) {
    $1=(dsid_t*)malloc(sizeof(dsid_t));
}

%typemap(in) dsid_t {
    if (!convert_uint8_t($input, (uint8_t*)&$1, sizeof(dsid_t))) {
        return NULL;
    }
}

%typemap(argout) (dsid_t *dsid) {
    convert_uint8_array_to_pylist(&$result, (uint8_t*)$1, sizeof(dsid_t));
    free($1);
}

%typemap(in,numinputs=0) (dsuid_t *dsuid)(uint8_t temp) {
    $1=(dsuid_t*)malloc(sizeof(dsuid_t));
}

%typemap(in) dsuid_t {
    if (!convert_uint8_t($input, (uint8_t*)&$1, sizeof(dsuid_t))) {
        return NULL;
    }
}

%typemap(argout) (dsuid_t *dsuid) {
    convert_uint8_array_to_pylist(&$result, (uint8_t*)$1, sizeof(dsuid_t));
    free($1);
}

%typemap(in) (dsuid_t *device_list, int device_list_size) {
    if (!PyInt_Check($input)) {
       PyErr_SetString(PyExc_ValueError, "Expecting an integer");
       return NULL;
   }
   $2 = PyInt_AsLong($input);
   if ($2 < 0) {
       PyErr_SetString(PyExc_ValueError, "Positive integer expected");
       return NULL;
   }
   $1 = (dsuid_t*)malloc(sizeof(dsuid_t) * $2);
}

%typemap(argout) (dsuid_t *device_list, int device_list_size) {
    PyObject *o, *o2, *o3;
    int i = 0;
    int num_results = PyInt_AsLong($result);
    for (i = 0; i < num_results; i++) {
        convert_uint8_array_to_pylist(&$result, (uint8_t*)((uint8_t*)$1 + (i * sizeof(dsuid_t))), sizeof(dsuid_t));
    }
    free($1);
}

%typemap(in,numinputs=0) bool *state(bool temp) {
    $1 = &temp;
}

%typemap(argout) bool *state {
    convert_uint8_to_pylong(&$result, (uint8_t*)$1);
}

%typemap(in) uint8_t *InName {
    $1 = (uint8_t*)malloc(21);
   if (!convert_string($input, $1, 21)) {
      return NULL;
   }
}

%typemap(freearg) uint8_t *InName {
   free($1);
}

%typemap(in,numinputs=0) uint8_t *OutName(uint8_t temp) {
   $1=(uint8_t*)malloc(21);
}

%typemap(argout) uint8_t *OutName {
    convert_uint8_attay_to_pystring(&$result, $1);
    free($1);
}

%typemap(in) uint8_t *InRequest {
   $1 = (uint8_t*)malloc(4096);
   if (!convert_uint8_t_max_size($input, $1, 4096)) {
      return NULL;
   }
}

%typemap(freearg) uint8_t *InRequest {
   free($1);
}

%typemap(in,numinputs=0) uint8_t *OutResponse(uint8_t temp) {
    $1=(uint8_t*)malloc(4096);
}

%typemap(argout)  (uint16_t *Out, uint8_t *OutResponse) {
    convert_uint8_array_to_pylist_max_size(&$result, $2, $1, 4096);
    free($2);
}

%typemap(in,numinputs=0) char** ($*1_type temp) {
   temp = NULL;
   $1=&temp;
}

%typemap(argout) char** {
    convert_uint8_attay_to_pystring(&$result, *$1);
}
"""
        if (len(length_enum) > 0):
            s = Template("""%typemap(in,numinputs=0) uint8_t *OutArray${length}(uint8_t temp) {
    $$1=(uint8_t*)malloc(${length});
}

%typemap(argout) uint8_t *OutArray${length} {
    convert_uint8_array_to_pylist(&$$result, $$1, ${length});
    free($$1);
}

%typemap(in) uint8_t *InArray${length} {
    $$1 = (uint8_t*)malloc(${length});
   if (!convert_uint8_t($$input, $$1, ${length})) {
      return NULL;
   }
}

%typemap(freearg) uint8_t *InArray${length} {
   free($$1);
}

%typemap(in) const uint8_t *InPayload {
    $$1 = (uint8_t*)malloc(64);
   if (!convert_uint8_t($$input, $$1, 64)) {
      return NULL;
   }
}

%typemap(freearg) const uint8_t *InPayload {
   free($$1);
}
""")
            swig_str += "".join(map(lambda x: s.substitute(length = str(x)), length_enum.itervalues()))

        for group in self.groups:
            swig_str += group.swig_declaration()
        
        swig_str += """%feature("autodoc", "DsmApiLibraryVersion() -> ApiVersion") DsmApiLibraryVersion;
%feature("docstring") "
Get the API version of the library.

This function returns the API version as two bytes. The first byte is the MAJOR version,
the second byte is the MINOR version. This version corresponds to the ApiVersion parameter read
from the dSM with #dSMInfo call. Differing MAJOR versions are binary incompatible. Higher MINOR
versions contain additional functionality not previously available.

Return values:
ApiVersion -- API version"
int DsmApiLibraryVersion(void);

%feature("autodoc", "DsmApiInitialize() -> handle") DsmApiInitialize;
%feature("docstring") "
Initialize the internal memory structures of the library.

Return values: 
handle -- a handle to the library instance"
DsmApiHandle_t DsmApiInitialize(void);

%feature("autodoc", "DsmApiOpen(handle, connection, flags) -> error") DsmApiOpen;
%feature("docstring") "Open a new connection to the dS485 bus.

Arguments:
handle -- Handle to the library instance
connection  -- string containing the connection method: 
                      - serial:///dev/ttyUSB0 (only when linked to the GPL version of the ds485-client library)
                      - tcp://localhost:8442
flags -- pass 0

Return values:
error -- zero on success, negative on failure"
int DsmApiOpen(DsmApiHandle_t handle, const char *connection, int flags);

%feature("autodoc", "DsmApiGetOwnDSUID(handle) -> (error, dsuid)") DsmApiGetOwnDSUID;
%feature("docstring") "Get the dSUID of the daeomon/socket interface.

Arguments:
handle -- Handle to the library instance

Return values:
error -- 0 on success; negative on failure
dsuid -- own dSUID"
int DsmApiGetOwnDSUID(DsmApiHandle_t handle, dsuid_t *dsuid);

%feature("autodoc", "DsmApiGetBusMembers(handle, device_list_size) -> (error, device_list)") DsmApiGetBusMembers;
%feature("docstring") "Request a list of the current bus members.

Arguments:
handle -- Handle to the library instance
device_list_size -- device_list has space for this many devices.

Return values:
error -- the number of devices in device_list; negative if an error occured
device_list -- List of devices. Preallocate a buffer of size device_list_size * sizeof(dsuid_t)"
int DsmApiGetBusMembers(DsmApiHandle_t handle, dsuid_t *device_list, int device_list_size);

%feature("autodoc", "DsmApiGetBusMembersPerType(handle, device_list_size, types) -> (error, device_list)") DsmApiGetBusMembersPerType;
%feature("docstring") "Request a list of the current bus members.

Arguments:
handle -- Handle to the library instance
device_list_size -- device_list has space for this many devices.
types -- type filter for device list.

Return values:
error -- the number of devices in device_list; negative if an error occured
device_list -- List of devices. Preallocate a buffer of size device_list_size * sizeof(dsuid_t)"
int DsmApiGetBusMembersPerType(DsmApiHandle_t handle, dsuid_t *device_list, int device_list_size, unsigned int types);

%feature("autodoc", "DsmApiGetLinkState(handle) -> (error, link)") DsmApiGetLinkState;
%feature("docstring") "Are we connected to a ds485 gateway

Arguments:
handle -- Handle to the library instance

Return values:
error -- 0 on success; negative on failure
link_state -- true if connected"
int DsmApiGetLinkState(DsmApiHandle_t handle, bool *state);

%feature("autodoc", "DsmApiRestartBus(handle) -> (error)") DsmApiRestartBus;
%feature("docstring") "Restart the dS485 bus.

Arguments:
handle -- Handle to the library instance

Return values:
error -- 0 on success; negative on failure"
int DsmApiRestartBus(DsmApiHandle_t handle);

%feature("autodoc", "DsmApiClose(handle) -> (error)") DsmApiClose;
%feature("docstring") "Close the connection to the dS485 bus.

Arguments:
handle -- Handle to the library instance

Return values:
error -- 0 on success; negative on failure"
int DsmApiClose(DsmApiHandle_t handle);

%feature("autodoc", "DsmApiCleanup(handle) -> (error)") DsmApiCleanup;
%feature("docstring") "Free instance memory.

Arguments:
handle -- Handle to the library instance

Return values:
error -- 0 on success; negative on failure"
int DsmApiCleanup(DsmApiHandle_t handle);

%feature("autodoc", "DsmApiRegisterPyCallback(handle, MessageType, MessageId, ModifierId, pyfunc) -> (error)") DsmApiRegisterPyCallback;
%feature("docstring") "Register callback functions for dsm-api asynchronous responses, requests and events.

Register NULL to delete a callback.

Arguments:
handle -- Handle to the library instance
MessageType -- Register callback function for this MessageType
MessageId -- Register callback function for this MessageId
ModifierId -- Register the callback function for thei ModifierId
pyfunc -- the callback function

Return values:
callback -- the previously registered callback function for this MessageId/ModifierId combination"
void DsmApiRegisterPyCallback(DsmApiHandle_t handle, container_type_t MessageType, int MessageId, int ModifierId, PyObject *pyfunc);

%feature("autodoc", "DsmApiRegisterPyBusChangeCallback(handle, pyfunc) -> (error)") DsmApiRegisterPyBusChangeCallback;
%feature("docstring") "Register a callback function for BusChange events.

Register NULL to delete the callback.

Arguments:
handle -- Handle to the library instance
pyfunc -- the callback function

Return values:
callback -- the previously registered callback function"
void DsmApiRegisterPyBusChangeCallback(DsmApiHandle_t handle, PyObject *pyfunc);

%feature("autodoc", "DsmApiRegisterPyLinkStateCallback(handle, pyfunc) ->
(error)") DsmApiRegisterPyLinkStateCallback;
%feature("docstring") "Register a callback function for LinkState events.

Register NULL to delete the callback.

Arguments:
handle -- Handle to the library instance
pyfunc -- the callback function

Return values:
callback -- the previously registered callback function"
void DsmApiRegisterPyLinkStateCallback(DsmApiHandle_t handle, PyObject *pyfunc);

%feature("autodoc", "DsmApiGetDeviceDSUID(vendorId, productId, groupId, revisionId, serialNumber) -> (error, dsuid)") DsmApiGetDeviceDSUID;
%feature("docstring") "Evaluate product properties and convert into a unique dSUID.

Arguments:
vendorId -- VendorId of the device
productId -- ProductId of the device
groupId -- GroupId of the device, the color or standard function
revisionId -- RevisionId of the device
serialNumber -- Serial number of the device

Return values:
error -- 0 on success, ERROR_DSUID_CONVERSION if VendorId is unknown (dSUID contains only serial number)
dsuid -- Contains the dSUID of the device."
int DsmApiGetDeviceDSUID(uint16_t vendorId, uint16_t productId, uint8_t groupId, uint16_t revisionId, uint32_t serialNumber, dsuid_t *dsuid);

%feature("autodoc", "DsmApiGetDeviceDescription(vendorId, productId, groupId, revisionId) -> (error, displayName, hardwareInfo, GTIN)") DsmApiGetDeviceDescription;
%feature("docstring") "Evaluate product properties and get product description strings

Arguments:
vendorId -- VendorId of the device
productId -- ProductId of the device
groupId -- GroupId of the device, the color or standard function
revisionId -- RevisionId of the device

Return values:
error -- 0 on success, ERROR_DSUID_CONVERSION if VendorId is unknown (dSUID contains only serial number)
displayName -- The user visible name of the device.
hardwareInfo -- The internally used name of the device.
GTIN -- The Global Trade Item Number of the device."
int DsmApiGetDeviceDescription(uint16_t vendorId, uint16_t productId, uint8_t groupId, uint16_t revisionId, char **displayName, char **hardwareInfo, char **GTIN);

%feature("autodoc", "DsmApiGetEthernetDSUID(mac) -> (error, dsuid)") DsmApiGetEthernetDSUID;
%feature("docstring") "Evaluate a MAC address and convert into a unique dSUID.

Arguments:
mac -- MAC Address

Return values:
error -- 0 on success, ERROR_DSUID_CONVERSION if VendorId is unknown (dSUID contains only serial number)
dsuid -- Contains the dSUID of the device."
int DsmApiGetEthernetDSUID(const char *mac, dsuid_t *dsuid);

%feature("autodoc", "DsmApiIsdSM(dSMdSUID) -> error") DsmApiIsdSM;

%feature("docstring") "Check if a given dSUID belongs to a dSM.

Arguments:
dSMdSUID -- dSUID of the device in question

Return values:
error -- 0: device is no dSM, 1: device is a dSM"
int DsmApiIsdSM(dsuid_t dSMdSUID);

%feature("autodoc", "DeviceStatisticsRequest_get_sync(handle, dSMdSUID, DeviceId, ParamOffset, timeout) -> (error, Value)") DeviceStatisticsRequest_get_sync;
%feature("docstring") "Requests statistics data of a device. Synchronous implementation of #DeviceStatisticsRequest_get.

ParamOffset 0xFE (Reset) and 0xFF (All values) are not allowed, as they would result in no or multiple response events.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId -- Device ID.
ParamOffset -- Index in statistics structure.
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
Value -- Statistics value (only valid if #error == 0)"
int DeviceStatisticsRequest_get_sync(DsmApiHandle_t handle, dsuid_t dsuid, uint16_t DeviceId, uint8_t ParamOffset, int timeout, uint16_t *Out);

%feature("autodoc", "DeviceConfig_get_sync_8(handle, dSMdSUID, DeviceId, ConfigClass, ConfigIndex, timeout) -> (error, Value)") DeviceConfig_get_sync_8;
%feature("docstring") "Read a configuration parameter of a specific digitalSTROM device. Synchronous implementation of #DeviceConfigRequest_get.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
ConfigClass --
ConfigIndex --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
Value -- Parameter value (only valid if return == 0)"
int DeviceConfig_get_sync_8(DsmApiHandle_t handle, dsuid_t dsuid, uint16_t DeviceId, uint8_t ConfigClass, uint8_t ConfigIndex, int timeout, uint8_t *Out);

%feature("autodoc", "DeviceConfig_get_sync_16(handle, dSMdSUID, DeviceId, ConfigClass, ConfigIndex, timeout) -> (error, Value)") DeviceConfig_get_sync_16;
%feature("docstring") "Read two configuration parameters of a specific digitalSTROM device simultaneously. Synchronous implementation of #DeviceConfigRequest_get.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
ConfigClass --
ConfigIndex --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
Value -- Parameter values low byte: ConfigIndex & 0xFE; high byte: ConfigIndex & 0xFE + 1 (only valid if return == 0)"
int DeviceConfig_get_sync_16(DsmApiHandle_t handle, dsuid_t dsuid, uint16_t DeviceId, uint8_t ConfigClass, uint8_t ConfigIndex, int timeout, uint16_t *Out);

%feature("autodoc", "DeviceConfig_set_sync(handle, dSMdSUID, DeviceId, ConfigClass, ConfigIndex, Value, timeout) -> (error)") DeviceConfig_set_sync;
%feature("docstring") "Set a configuration parameter of a specific digitalSTROM device. Synchronous implementation of #DeviceConfig_set.

Caution: This function doesn't guarantee that the device received the value. On success, it just means that the dSM could sent the Value in given time.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
ConfigClass --
ConfigIndex --
Value --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code"
int DeviceConfig_set_sync(DsmApiHandle_t handle, dsuid_t dsuid, uint16_t DeviceId, uint8_t ConfigClass, uint8_t ConfigIndex, uint8_t Value, int timeout);

%feature("autodoc", "DeviceProperties_set_zone_sync(handle, dSMdSUID, DeviceId, ZoneId, timeout) -> (error)") DeviceProperties_set_zone_sync;
%feature("docstring") "Set a zone of a specific digitalSTROM device. Synchronous implementation of #DeviceProperties_set_zone.

Caution: This function doesn't guarantee that the device received the value. On success, it just means that the dSM could sent the ZoneId in given time.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
ZoneId --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code"
int DeviceProperties_set_zone_sync(DsmApiHandle_t handle, dsuid_t dsuid, uint16_t DeviceId, uint16_t ZoneId, int timeout);

%feature("autodoc", "DeviceProperties_set_button_set_output_channel_sync(handle, dSMdSUID, DeviceId, ChannelId, timeout) -> (error)") DeviceProperties_set_button_set_output_channel_sync;
%feature("docstring") "Set a button output channel of a specific digitalSTROM device. Synchronous implementation of #DeviceProperties_set_button_set_output_channel.

Caution: This function doesn't guarantee that the device received the value. On success, it just means that the dSM could sent the ChannelId in given time.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
ChannelId --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code"
int DeviceProperties_set_button_set_output_channel_sync(DsmApiHandle_t handle, dsuid_t dsuid, uint16_t DeviceId, uint8_t ChannelId, int timeout);

%feature("autodoc", "DeviceProperties_set_button_set_global_group_sync(handle, dSMdSUID, DeviceId, GroupId, timeout) -> (error)") DeviceProperties_set_button_set_global_group_sync;
%feature("docstring") "Set a button global group of a specific digitalSTROM device. Synchronous implementation of #DeviceProperties_set_button_set_global_group.

Caution: This function doesn't guarantee that the device received the value. On success, it just means that the dSM could sent the GroupId in given time.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
GroupId --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code"
int DeviceProperties_set_button_set_global_group_sync(DsmApiHandle_t handle, dsuid_t dsuid, uint16_t DeviceId, uint8_t GroupId, int timeout);

%feature("autodoc", "DeviceGroupMembershipModify_add_sync(handle, dSMdSUID, DeviceId, GroupId, timeout) -> (error)") DeviceGroupMembershipModify_add_sync;
%feature("docstring") "Add a group to a specific digitalSTROM device. Synchronous implementation of #DeviceGroupMembershipModify_add.

Caution: This function doesn't guarantee that the device received the value. On success, it just means that the dSM could sent the GroupId in given time.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
GroupId --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code"
int DeviceGroupMembershipModify_add_sync(DsmApiHandle_t handle, dsuid_t dsuid, uint16_t DeviceId, uint8_t GroupId, int timeout);

%feature("autodoc", "DeviceOPCConfig_get_current_sync(handle, dSMdSUID, DeviceId, ChannelId, timeout) -> (error, Value)") DeviceOPCConfig_get_current_sync;
%feature("docstring") "Read the current value of an output channel of a digitalSTROM device. Synchronous implementation of #DeviceOPCConfig_get_current.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
ChannelId --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
Value -- Parameter values (only valid if return == 0)"
int DeviceOPCConfig_get_current_sync(DsmApiHandle_t handle, dsuid_t dSMdSUID, uint16_t DeviceId, uint8_t ChannelId, int timeout, uint16_t *Out);

%feature("autodoc", "DeviceOPCConfig_get_scene_sync(handle, dSMdSUID, DeviceId, ChannelId, SceneId, timeout) -> (error, Value)") DeviceOPCConfig_get_scene_sync;
%feature("docstring") "Read the scene value of an output channel of a digitalSTROM device. Synchronous implementation of #DeviceOPCConfig_get_scene.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
ChannelId --
SceneId --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
Value -- Parameter values (only valid if return == 0)"
int DeviceOPCConfig_get_scene_sync(DsmApiHandle_t handle, dsuid_t dSMdSUID, uint16_t DeviceId, uint8_t ChannelId, uint8_t SceneId, int timeout, uint16_t *Out);

%feature("autodoc", "DeviceOPCConfig_get_scene_config_sync(handle, dSMdSUID, DeviceId, SceneId, timeout) -> (error, Value)") DeviceOPCConfig_get_scene_config_sync;
%feature("docstring") "Read the scene config value for a scene of a digitalSTROM device. Synchronous implementation of #DeviceOPCConfig_get_scene_config.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
SceneId --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
Value -- Parameter values (only valid if return == 0)"
int DeviceOPCConfig_get_scene_config_sync(DsmApiHandle_t handle, dsuid_t dSMdSUID, uint16_t DeviceId, uint8_t SceneId, int timeout, uint16_t *Out);

%feature("autodoc", "DeviceOPCConfig_get_dc_flags_sync(handle, dSMdSUID, DeviceId, SceneId, timeout) -> (error, Value)") DeviceOPCConfig_get_dc_flags_sync;
%feature("docstring") "Read don't care flags bitfield for a scene of a digitalSTROM device. Synchronous implementation of #DeviceOPCConfig_get_dc_flags.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId --
SceneId --
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
Value -- Parameter values (only valid if return == 0)"
int DeviceOPCConfig_get_dc_flags_sync(DsmApiHandle_t handle, dsuid_t dSMdSUID, uint16_t DeviceId, uint8_t SceneId, int timeout, uint16_t *Out);

%feature("autodoc", "TestTransmissionQuality_get_sync(handle, dSMdSUID, DeviceId, timeout) -> (error, DownstreamQuality, UpstreamQuality)") TestTransmissionQuality_get_sync;
%feature("docstring") "Query transmission quality. Synchronous implementation of #TestTransmissionQuality_get.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId -- Device ID.
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
DownstreamQuality -- Downstream quality (only valid if return == 0)
UpstreamQuality -- Upstream quality (only valid if return == 0)"
int TestTransmissionQuality_get_sync(DsmApiHandle_t handle, dsuid_t dSMdSUID, uint16_t DeviceId, int timeout, uint8_t *Out, uint16_t *Out);

%feature("autodoc", "DeviceSensor_get_value_sync(handle, dSMdSUID, DeviceId, SensorIndex, timeout) -> (error, SensorValue)") DeviceSensor_get_value_sync;
%feature("docstring") "Access device sensors. Get the value of sensor SensorIndex. Synchronous implementation of #DeviceSensor_get_value.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId -- DeviceId
SensorIndex -- Index of the requested sensor on the device.
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
SensorValue -- Current value of the sensor (12 bit)."
int DeviceSensor_get_value_sync(DsmApiHandle_t handle, dsuid_t dSMdSUID, uint16_t DeviceId, uint8_t SensorIndex, int timeout, uint16_t *Out);

%feature("autodoc", "DeviceDslinkSend_not_last_byte_with_answer_sync(handle, dSMdSUID, DeviceId, Value, timeout) -> (error, ReturnValue)") DeviceDslinkSend_not_last_byte_with_answer_sync;
%feature("docstring") "Send UDI commands to a single dSLink device. UDI device stays selected after the call execution. Used when multiple bytes have to be sent. Synchronous implementation of #DeviceDslinkSend_not_last_byte_with_answer.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId -- Select only this device.
Value -- Value to send to devices.
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
ReturnValue -- Answer from the device"
int DeviceDslinkSend_not_last_byte_with_answer_sync(DsmApiHandle_t handle, dsuid_t dSMdSUID, uint16_t DeviceId, uint8_t Value, int timeout, uint8_t *Out);

%feature("autodoc", "DeviceDslinkSend_last_byte_with_answer_sync(handle, dSMdSUID, DeviceId, Value, timeout) -> (error, ReturnValue)") DeviceDslinkSend_last_byte_with_answer_sync;
%feature("docstring") "Send UDI commands to a single dSLink device. UDI device is de-selected after the call. Used only on the last access. Synchronous implementation of #DeviceDslinkSend_last_byte_with_answer.

Arguments:
handle -- Handle to the library instance
dSMdSUID -- Destination of the call
DeviceId -- Select only this device.
Value -- Value to send to devices.
timeout -- time-out of the synchronous call (in Seconds)

Return values:
error -- Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
ReturnValue -- Answer from the device"
int DeviceDslinkSend_last_byte_with_answer_sync(DsmApiHandle_t handle, dsuid_t dSMdSUID, uint16_t DeviceId, uint8_t Value, int timeout, uint8_t *Out);

typedef enum {
    DS485_CONTAINER_REQUEST = 1,
    DS485_CONTAINER_RESPONSE,
    DS485_CONTAINER_EVENT
} container_type_t;

typedef void *DsmApiHandle_t;
%include "dsm-api-const.h"
"""
        swig.write(swig_str.encode('utf-8'))
        swig.close()
        
